# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#

"""
Helper function to compile latex file in a temp dir
"""

import logging as log
import re
import shutil
import subprocess
import sys
from pathlib import Path

from rich.live import Live
from rich.spinner import Spinner

# Match only lines beginning with INPUT
INPUT_RE = re.compile(r"^INPUT")
INPUT_STY = re.compile(r"^INPUT\s.*.(cls|def|sty)")
INPUT_FONTS = re.compile(r"^INPUT\s.*.(pfb|tfm)")
FONT_PUBLIC = re.compile(r"/public/.*/")


def run_latexmk(filename, mode, compdir, timeout=300, silent: bool = False):
    """Helper function to actually compile the latex file in a tmpdir"""
    latexmk_bin = shutil.which("latexmk")
    if not latexmk_bin:
        log.critical("Unable to find `latexmk` in PATH.")
        sys.exit(1)

    # Generate fls file from tex file by running latexmk
    latexmk_cmd = [
        latexmk_bin,
        f"-{mode}",
        "-f",
        "-cd",
        f"-outdir={compdir}",
        "-interaction=nonstopmode",
        filename.name,
    ]
    try:
        with Live(
            Spinner("dots2", text="Compiling LaTeX project"),
            transient=True,
        ):
            subprocess.run(
                latexmk_cmd,
                capture_output=True,
                encoding="utf-8",
                check=True,
                timeout=timeout,
            )
    except subprocess.TimeoutExpired as e:
        log.critical(
            "Error: %s process timed out after %s seconds",
            latexmk_bin,
            timeout,
        )
        raise e
    except OSError as err:
        log.error("%s", err.strerror)
        raise err
    except subprocess.CalledProcessError as err:
        log.critical(f"{err.cmd[0]} failed to compile project.")
        log.info(
            "Error: %s failed with the following output:\n%s\n%s",
            err.cmd[0],
            err.stdout,
            "===================================================",
        )
        log.info("Latexmk command used was: `%s`", " ".join(latexmk_cmd))
        raise err

    if not silent:
        log.info(
            "LaTeX project successfully compiled with: %s",
            " ".join(latexmk_cmd),
        )
    fls_path = Path(compdir) / f"{filename.stem}.fls"
    log.debug("%s generated", fls_path.as_posix())
    return fls_path


def fls_input_files(fls_fileobj, skip_files, *, sty_files=False):
    """Helper function to return list on files marked as 'INPUT' in fls file"""
    deps = set()
    pkgs = {"System": set(), "Local": set()}
    for line in fls_fileobj:
        if INPUT_RE.match(line):
            p = Path(line.split()[-1])
            if not p.is_absolute() and (p.suffix not in skip_files):
                deps.add(p)

        if sty_files:
            if INPUT_STY.match(line):
                p = Path(line.split()[-1])
                if p.is_absolute():
                    # Base is not a (La)TeX package; it is installed with even
                    # the most basic TeXlive/MikTeX installation
                    if (pdir := p.parent.name) != "base":
                        pkgs["System"].add(pdir)
                else:
                    pkgs["Local"].add(p.name)
            elif INPUT_FONTS.match(line):
                p = Path(line.split()[-1])
                if p.is_absolute():
                    _mat = FONT_PUBLIC.search(str(p))
                    fontdir = (
                        _mat.group(0).split("/")[2] if _mat else p.parent.name
                    )
                    pkgs["System"].add(fontdir)

    return deps, pkgs



# vim:set et sw=4 ts=4 tw=80:
# module _tar
"""
Helper class TarFiles
"""

import logging as log
import math
import os
from io import BytesIO
from pathlib import Path
from rich import print as richprint
from .utils.tar_utils import TAR_EXT
from .utils.msg_utils import summary_msg
import tarfile as tar


class Tarballer:
    """Class that handles tarballing a list of objects (file Paths, BytesIO, etc.)"""

    def __init__(
        self, curr_dir: Path, main_input_file: Path, target: Path = Path(".")
    ):
        """Init class for TarFiles"""

        # set of actual, accessible files to include
        self._files: set[Path] = set()

        # dict of objects to include whose contents are only available as BytesIO
        # key: name to use; val: object contents as BytesIO
        self._streams: dict[str, BytesIO] = {}

        # dict of strings to use for logging when adding `key` object to tarball
        # key: file or stream object name; val: logging string
        self._comments: dict[str, str] = {}

        self._main_file: Path = main_input_file
        self._mtime = os.path.getmtime(self._main_file)

        self._work_dir: Path = curr_dir
        target_path: Path = self._work_dir / target
        self._target: Path
        self._ext = target_path.suffix.lstrip(".")
        self._target = target_path.with_suffix(f".{self._ext}")

    @property
    def num_objects(self) -> int:
        """Returns the total number of files and bytes objects added to tarball
        :returns: int

        """
        return len(self._files) + len(self._streams)

    def recomp_mode(self, recomp: str):
        """Set re-compression mode for tarball

        :recomp: one of "bz2", "gz", or "xz" (str)
        :returns: None

        """
        self._ext = recomp if recomp in TAR_EXT else "gz"
        self._target = self._target.with_suffix(self._ext)

    def set_mtime(self, mtime: float):
        """Set mtime for bytesio objects, for which mtime is not available otherwise"""
        self._mtime = mtime

    def app_files(self, *args: Path, comm: str = ""):
        """Update set of files to add to tarball with args

        :*args: files to append to `self._files` as args
        :returns: None

        """
        self._files.update(args)
        for f in args:
            if comm:
                self._comments[f.as_posix()] = comm

    def drop_files(self, *args: Path):
        """Drop files and associated comments, if any"""
        for f in args:
            self._files.discard(f)
            _ = self._comments.pop(f.as_posix(), None)

    def files(self) -> set[Path]:
        """Return copy of self._files as a set of Path(s)
        :returns: set of Paths

        """
        return self._files.copy()

    def streams(self) -> set[str]:
        """Return copy of self._streams.keys() as a set of strings
        :returns: set of str

        """
        return set(self._streams.keys())

    def objects(self) -> set[Path]:
        """Return all files and streams to be added to tarball"""
        return self.files().union([Path(f) for f in self.streams()])

    def app_stream(self, name: str, content: BytesIO, comm: str = ""):
        """Append a single BytesIO content to list of streams

        :name: file name to use for stream (str)
        :content: stream content corresponding to `name` (BytesIO)
        :returns: None

        """
        self._streams[name] = content
        if comm:
            self._comments[name] = comm

    def do_tar(self):
        def _tar_add_file(
            tar_obj: tar.TarFile,
            file_name: Path,
        ):  # helper func to add file <file_name> to <tar_obj>
            tinfo = tar_obj.gettarinfo(file_name)
            tinfo.uid = tinfo.gid = 0
            tinfo.uname = tinfo.gname = ""
            tar_obj.addfile(tinfo, open(file_name, "rb"))
            log.info("Add file: %s", file_name)

        def _tar_add_bytesio(tar_obj: tar.TarFile, file_name: str, obj):
            # helper func to add `obj` as BytesIO with filename <file_name> to <tar_obj>
            tinfo = tar_obj.tarinfo(file_name)
            tinfo.size = len(obj)
            tinfo.mtime = self._mtime
            tinfo.uid = tinfo.gid = 0
            tinfo.uname = tinfo.gname = ""
            tar_obj.addfile(tinfo, BytesIO(obj))
            log.info("Add contents as BytesIO: %s", file_name)

        # At this point, output tar name conflicts, if any, has been
        # resolved one way or another, and we should simply over-write
        # tarball if it exists. So, it is safe to use 'w:'
        try:
            lof = self._files.copy()
            with tar.open(self._target, mode=f"w:{self._ext}") as tar_obj:
                for dep in lof:
                    try:
                        _tar_add_file(tar_obj, dep)
                    except FileNotFoundError:
                        log.warning(
                            "Skip missing INPUT file '%s'; try"
                            " forcing a LaTeX recompile ('-F').",
                            dep,
                        )
                        self._files.discard(dep)
                        continue
                for key, val in self._streams.items():
                    _tar_add_bytesio(tar_obj, key, val)
        except PermissionError as e:
            log.critical(e)
            raise e
        except Exception as e:
            raise e

    def print_list(self, _summ: bool = False):
        """helper function to print list of files in a pretty format"""
        total = len(self._files) + len(self._streams)
        idx_width = int(math.log10(total)) + 1  # num of chars for serial nums
        files_cnt = 0
        for f in sorted(self._files):
            if f.exists():
                richprint(f"{files_cnt + 1:{idx_width}}.", end=" ")
                print(str(f))
                files_cnt += 1
            else:
                log.warning("Missing file skipped: %s", f)
                total -= 1
        for r in sorted(self._streams.keys()):
            richprint(f"{'*':>{idx_width + 1}}", end=" ")
            print(f"{r}")
        if _summ:
            summary_msg(total)



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#

import sys
from pathlib import Path
import logging as log
from rich import print as richprint
from rich.prompt import Prompt

# Allowed tar extensions
TAR_EXT = ["bz2", "gz", "xz"]

# Default compression
TAR_DEFAULT_COMP = "gz"


def strip_tarext(filename: Path):
    """Strip '.tar(.EXT)' from filename"""
    while filename.suffix.lstrip(".") in TAR_EXT + ["tar"]:
        filename = filename.with_suffix("")
    return filename


def proc_output_path(
    wdir: Path, main_file: Path, user_path: Path, git_tag: str = ""
):
    """
    Returns the output tar file path (sans any '.tar.?z' suffix) after
    resolving the value passed to '--output' as part of cmdline options.

    Also sets the tar ext if determined from '--output' argument.

    Uses `user_path` instead of the value of `--output` argument if passed.
    """

    # If self.args.output is absolute, '/' simply returns it as a PosixPath
    out = (wdir / user_path.expanduser()).resolve()
    tar_ext = ""

    if out.is_dir():  # If dir, set to DIR/${main_file}.tar.gz
        log.debug("%s is an existing dir", out)
        out = out / (f"{main_file.stem}{git_tag}")
    elif (ext := out.suffix.lstrip(".")) in TAR_EXT:
        tar_ext = ext
    else:
        out = out.with_name(out.name)

    out = strip_tarext(out)
    log.debug("Processed output target basename: %s", out)
    return out, tar_ext


def tar_name_conflict(
    wdir: Path,  # calling dir
    filename: Path,  # main .tex/.fls filename
    tpath: str,  # full path to initial tarball name
    overwrite: bool = False,  # whether to overwrite tarball or not
    git_tag: str = "",  # full tag to be used for git-rev
):
    """
    Resolves conflict in case output tarball file already exists by either of
    the following options offered to user:

    * Overwriting existing file (automatically chosen if `overwrite` is
      `True`).
    * Asking for new tarball name (if tarball with new name already exists,
      exit with error).
    * Quit directly.

    :wdir: dir from which `tar_files()` is called (Path)
    :filename: main .tex/.fls file path (Path)
    :tpath: full filename of initial tarball (str)
    :overwrite: whether to overwrite existing tarball (optional; bool)
    :git_tag: full tag to be appended to tarball filename when `git-rev` is
    used (optional; str)

    :returns: new_tar_file and new extension if any (otherwise "") (tuple)
    """
    ext = Path(tpath).suffix.lstrip(".")

    if overwrite:
        log.warning(f"Overwriting existing tar file {tpath}")
        return tpath, ext
    richprint(
        "[bold red]A tar file with the same name"
        rf" \[{wdir / tpath}]"
        " already exists[/bold red]"
    )

    ocq = Prompt.ask(
        "What would you like to do "
        r"([bold blue]\[O][/bold blue]verwrite /"
        r" [bold green]\[C][/bold green]hoose new name /"
        r" *[bold red]\[Q][/bold red]uit)?"
    )
    if ocq.lower() in ["", "q"]:
        richprint(
            "[bold]Not overwriting existing tar file[/bold]\nQuitting",
            file=sys.stderr,
        )
        sys.exit(1)
    elif ocq.lower() == "c":
        new_name = Path(
            Prompt.ask("Enter [bold]new name[/bold] for tar file")
        ).expanduser()
        new_path, new_ext = proc_output_path(wdir, filename, new_name, git_tag)
        if new_ext:
            ext = new_ext
        new_path = Path(f"{new_path!s}.tar.{ext}").resolve()

        if new_path == tpath:
            richprint(
                "[bold red]Error: New name entered is also the same as"
                " existing tar file name[/bold red]\nQuitting",
                file=sys.stderr,
            )
            sys.exit(1)
        elif new_path.exists():
            richprint(
                "[bold red]Error: A tar file with the same name"
                rf" \[{(wdir / new_path)!s}] also"
                " exists[/bold red]\nQuitting",
                file=sys.stderr,
            )
            sys.exit(1)
        else:
            log.info("Tar file %s will be generated", new_path.as_posix())
            return new_path, ext
    elif ocq.lower() == "o":
        log.warning(f"Overwriting existing tar file {tpath}")
        return tpath, ext
    else:
        richprint(
            "[bold red]Error: Invalid response[/]\nQuitting.",
            file=sys.stderr,
        )
        sys.exit(1)



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT

"""Tests for different tar compression methods"""

from pathlib import Path
import pytest
import tempfile

from tartex.tartex import TarTeX
from tartex.utils.tar_utils import TAR_DEFAULT_COMP


@pytest.fixture
def target_tar(monkeypatch_set_main_file, monkeypatch_mtime):
    # See conftest.py for explanation of monkeypatching _set_main_file
    monkeypatch_set_main_file("some_file.tex")
    monkeypatch_mtime("some_file.tex")

    def _target(tar_ext, cmp_opt=""):
        ttx_opts = ["-v", "-s", "-o", f"dest.tar.{tar_ext}", "some_file.tex"]
        if cmp_opt:
            ttx_opts.append(cmp_opt)
        return TarTeX(ttx_opts)

    return _target


class TestTarExt:
    """
    Class of tests checking automatic tar compression detection based on user
    specified outout tarfile name
    """

    def test_default_compress(self, sample_texfile):
        """Test default tarball extension"""
        assert sample_texfile.tar_ext == TAR_DEFAULT_COMP

    def test_output_file_compress(self, target_tar):
        """Test user output file for different tar.foo extensions"""
        # bz2
        assert target_tar("bz2").tar_ext == "bz2"
        # tar.bz2 overridden by "-J"
        assert target_tar("bz2", "-J").tar_ext != "bz2"
        # xz
        assert target_tar("xz").tar_ext == "xz"
        assert (
            target_tar("xz").tar_file_w_ext.with_suffix("").name == "dest.tar"
        )


class TestTarDir:
    """Class of tests checking user-entered dir as output path"""

    main_file = "some_file.tex"
    tartex_args = [main_file, "-s", "-v", "-o"]

    def test_output_dir(self, monkeypatch_set_main_file, monkeypatch_mtime):
        """Test tarball output when -o is existing dir"""
        monkeypatch_set_main_file(self.main_file)
        monkeypatch_mtime(self.main_file)

        with tempfile.TemporaryDirectory() as tmpdir:
            t = TarTeX([*self.tartex_args, f"{tmpdir!s}"])

            assert t.tar_file_w_ext == (
                (Path(tmpdir) / self.main_file).with_suffix(".tar.gz")
            )

    def test_output_dir_filename(
        self, monkeypatch_set_main_file, monkeypatch_mtime
    ):
        """
        Test tarball output when -o is file name in existing dir (default
        compression)
        """
        monkeypatch_set_main_file(self.main_file)
        monkeypatch_mtime(self.main_file)

        with tempfile.TemporaryDirectory() as tmpdir:
            t = TarTeX([*self.tartex_args, f"{tmpdir!s}/foo"])

            assert t.tar_file_w_ext == (
                (Path(tmpdir) / "foo").with_suffix(".tar.gz")
            )

    def test_output_dir_xz(self, monkeypatch_set_main_file, monkeypatch_mtime):
        """
        Test tarball output when -o is dir but '-J' is passed
        for xz compression
        """
        monkeypatch_set_main_file(self.main_file)
        monkeypatch_mtime(self.main_file)

        with tempfile.TemporaryDirectory() as tmpdir:
            t = TarTeX([*self.tartex_args, f"{tmpdir!s}", "-J"])

            assert t.tar_file_w_ext == (
                Path(tmpdir) / self.main_file
            ).with_suffix(".tar.xz")



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#

"""
Tests for when source dir has .fls file
"""

import tarfile as tar

import pytest

from tartex.tartex import TarTeX


@pytest.fixture
def flsfile():
    """Default fls file name"""
    return "main.fls"


@pytest.fixture
def tartex_obj(datadir, flsfile):
    """Default tartex object with flsfile as input"""
    return TarTeX(
        [str(datadir / flsfile), "-o", str(datadir), "-bs"],
    )


class TestUseFls:
    """Tests for using in-project fls file as main input file"""

    def test_fls_main_arg(self, tartex_obj, flsfile):
        """User passes a .fls filename as input"""
        t = tartex_obj
        assert t.tar_file_w_ext.stem == flsfile.replace(".fls", ".tar")

    def test_fls_main_arg_noext(self, datadir, flsfile):
        """
        User passes a file name with no extension but tartex should find .fls
        file
        """
        flsfile_noext = flsfile.removesuffix(".fls")
        t = TarTeX(
            [str(datadir / flsfile_noext), "-o", str(datadir), "-b"],
        )
        t.tar_files()
        assert t.tar_file_w_ext.name == f"{flsfile_noext}.tar.{t.tar_ext}"
        with tar.open(f"{t.tar_file_w_ext}") as f:
            assert len(f.getnames()) == 2

    def test_fls_missing_bbl(self, tartex_obj, flsfile):
        """
        Verify that missing .bbl file is omitted from tarball and the call to
        tar_files() does not cause an exception
        """
        tartex_obj.tar_files()

        with tar.open(f"{tartex_obj.tar_file_w_ext}") as f:
            assert flsfile.replace(".fls", ".bbl") not in f.getnames()

    def test_no_permission(self, datadir, flsfile, caplog):
        """
        Setting output to a dir without write perms will print appropriate
        msg
        """
        t = TarTeX(
            [str(datadir / flsfile), "-o", "/test_no_perms.tar.bz2", "-v"]
        )
        with pytest.raises(SystemExit) as exc:
            t.tar_files()

        assert exc.value.code == 1
        err_msg = caplog.text.lower()
        assert "critical" in err_msg
        assert "permission denied" in err_msg

    def test_fls_recompile(self, datadir, flsfile):
        """Forcing a LaTeX recompile ensures .bbl is included"""
        t = TarTeX([str(datadir / flsfile), "-o", str(datadir), "-b", "-F"])
        t.tar_files()

        with tar.open(f"{t.tar_file_w_ext}") as f:
            assert flsfile.replace(".fls", ".bbl") in f.getnames()

    def test_fls_no_pdf(self, datadir, flsfile, caplog):
        """Silently skip missing pdf when `--with-pdf is used`"""
        t = TarTeX(
            [
                str(datadir / flsfile),
                "-o",
                str(datadir / "missing_pdf.tar.xz"),
                "-v",
                "--with-pdf",
            ]
        )
        t.tar_files()
        assert t.tar_file_w_ext.exists()
        with tar.open(f"{t.tar_file_w_ext}") as f:
            assert t.main_file.with_suffix(".pdf") not in f.getnames()
        assert "Skipping pdf not found" in caplog.text


class TestFlsNoCache:
    """Check that no cache is touched when using .fls file directly"""

    def test_fls_no_cache(self, datadir, flsfile):
        """
        cache file must not be generated when using .fls directly
        """
        flsfile_noext = flsfile.removesuffix(".fls")
        t = TarTeX(
            [
                str(datadir / flsfile_noext),
                "-o",
                str(datadir / "fls_no_cache"),
                "-b",
            ],
        )
        t.tar_files()
        assert not t.filehash_cache.exists()



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"""Tests for tarball generation from basic latex files"""

from pathlib import Path

import pytest

from tartex.tartex import TarTeX


@pytest.fixture
def default_target(datadir):
    return Path(datadir) / "test"


@pytest.fixture
def default_tartex_obj(datadir, default_target):
    return lambda fname: TarTeX(
        [
            (Path(datadir) / fname).as_posix(),
            "-v",
            "-s",
            "-o",
            default_target.as_posix(),
        ]
    )


class TestLaTeXmkFail:
    """Tests checking error reporting and clean up when latexmk fails"""

    def test_cleanup(self, default_target, default_tartex_obj, capsys):
        """Should clean up tarball when latexmk fails"""
        t = default_tartex_obj("error_latex.tex")
        with pytest.raises(SystemExit) as exc:
            t.tar_files()

        assert exc.value.code == 1
        assert not t.tar_file_w_ext.exists()

    def test_err_msg(self, default_target, datadir, capsys, join_linebreaks):
        """Should print an error msg when latexmk fails"""
        t = TarTeX(
            [
                (Path(datadir) / "error_latex.tex").as_posix(),
                "-vv",
                "-s",
                "-o",
                default_target.with_suffix(".xz").as_posix(),
            ]
        )
        with pytest.raises(SystemExit) as exc:
            t.tar_files()
            assert (
                "latexmk failed with the following output"
                in join_linebreaks(capsys.readouterr().err)
            )
            assert "command used was:" in capsys.readouterr().err
            assert "Cleaning up empty tarball" in join_linebreaks(
                capsys.readouterr().err
            )

        assert exc.value.code == 1



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"Helper module for XDG based cache dir discovery"

from .hash_utils import HASH_METHOD, SHORT_HASH_LEN
import os
from pathlib import Path
from tartex.__about__ import __appname__

XDG_CACHE_HOME = Path(os.getenv("XDG_CACHE_HOME") or Path.home() / ".cache")


def app_cache_dir(main_file: Path) -> Path:
    """
    Set up and return path to unique cache dir to be used for `main_file`.
    Every `main_file` must have its own unique cache dir inside the global
    cache path `${XDG_CACHE_HOME}/tartex/`.

    :main_file: main .tex|.fls file [Path]
    :returns: Path to cache dir [Path]

    """
    cache_dir = XDG_CACHE_HOME / __appname__
    prj_dir = main_file.resolve().parent

    # Use bottom two parent dirs and join them using "_"
    path_parts = prj_dir.parts
    if len(path_parts) >= 2:
        dir_short_name = "_".join(str(prj_dir).split(os.sep)[-2:])
    else:
        dir_short_name = prj_dir.name
    # Append a short dir hash making the dir name unique
    dir_short_hash = HASH_METHOD(
        str(main_file.resolve()).encode("utf-8")
    ).hexdigest()[:SHORT_HASH_LEN]
    prj_cache_dir = cache_dir / (f"{dir_short_name}_{dir_short_hash}")
    prj_cache_dir.mkdir(parents=True, exist_ok=True)
    return prj_cache_dir



# vim:set et sw=4 ts=4 tw=80:
# SPDX-FileCopyrightText: 2024-present Atri Bhattacharya <atrib@duck.com>
#
# SPDX-License-Identifier: MIT
#
"""utilities for saving and checking file hash"""

import hashlib
import json
import logging as log
from pathlib import Path
from .tex_utils import SetEncoder, SUPP_REQ

HASH_METHOD = hashlib.sha256

SHORT_HASH_LEN = 7

def save_input_files_hash(
    cache_file: Path,
    files: set[Path],
    streams: set[str],
    packages: dict[str, set[str]],
) -> None:
    hash_dict: dict[str, str] = {}
    for p in files:
        try:
            with open(p, mode="rb") as f:
                hash_dict[str(p)] = hashlib.sha256(f.read()).hexdigest()
        except FileNotFoundError:
            continue

    _cache = {
        "input_files": hash_dict,
        "streams": streams,
        "packages": packages,
    }
    with open(cache_file, mode="w") as f:
        json.dump(
            _cache,
            f,
            cls=SetEncoder,
            indent=4,
            ensure_ascii=True,
        )
    try:
        c_path_msg = f"~/{cache_file.relative_to(Path.home())}"
    except ValueError:
        c_path_msg = str(cache_file)
    log.info("Updated cache: %s", c_path_msg)
    return


def check_file_hash(cache_file: Path) -> bool:
    try:
        with open(cache_file, mode="r") as cf:
            cache_dict = json.load(cf)
    except Exception:
        return False

    # if any supplementary files are found as `streams` in the cache file, this
    # implies they are missing from the project dir and a recompile may be
    # required (otherwise file will be missing from tarball too). Log info.
    for supp in cache_dict["streams"]:
        if Path(supp).suffix.lstrip(".") in SUPP_REQ:
            log.info(
                "Missing supplementary file %s, try recompile ('-F')", supp
            )

    for filename in cache_dict["input_files"].keys():
        try:
            with open(filename, mode="rb") as _f:
                if (
                    cache_dict["input_files"][filename]
                    != hashlib.sha256(_f.read()).hexdigest()
                ):
                    return False
        except FileNotFoundError as err:
            log.warning("Unable to find file: %s", err.filename)
            return False
    else:
        return True
